#
# (c) 2002 Andreas Kostyrka
#

from OFS import Folder
from AccessControl import ClassSecurityInfo, getSecurityManager, Role
from Globals import DTMLFile
from Acquisition import aq_parent,aq_base,Implicit,ImplicitAcquisitionWrapper,aq_inner
from Globals import MessageDialog
import Globals
from AccessControl.Role import RoleManager
from Products.ZCatalog.CatalogPathAwareness import CatalogPathAware
import sys
from zLOG import *

addFlexData=DTMLFile('dtml/FlexDataAdd',globals())

def manage_addFlexData(self,id,subtype,REQUEST=None):
    "add a FlexData object."
    ob=FlexData()
    ob.id=id
    ob.subtype=subtype
    self._setObject(id, ob)
    ob=self._getOb(id)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class DummySubtypeClass(Implicit):
    pass

class FlexData(CatalogPathAware,Folder.Folder):
    meta_type='FlexData'

    security=ClassSecurityInfo()
    security.declareProtected("Manage properties",
                              "manage_addProperty","manage_delProperties",
                              "manage_changeProperties",
                              "manage_editProperties")

    _properties=(
        {'id':'subtype','type':'string','mode':'w'},
        )

    manage_options=(
        Folder.Folder.manage_options+
        ({'label':'Catalog','action':'manage_editCatalogerForm'},))


    ## This makes the Property edit interface update the catalog data.
    def manage_editProperties(self,REQUEST):
        "edit the properties tru the web and reindex the object."
        r=Folder.Folder.manage_editProperties(self,REQUEST)
        self.index_object()
        return r


    def manage_changeProperties(self,REQUEST=None,**kw):
        "edit the properties tru the web and reindex the object."
        r=Folder.Folder.manage_changeProperties(self,REQUEST,**kw)
        self.index_object()
        return r


    def manage_addProperty(self, id, value, type, REQUEST=None):
        "Add a new property"
        r=Folder.Folder.manage_addProperty(self,id,value,type,REQUEST)
        self.index_object()
        return r

    def manage_delProperties(self, ids=None, REQUEST=None):
        "delete properties"
        r=Folder.Folder.manage_delProperties(self,ids,REQUEST)
        self.index_object()
        return r

    def __of__(self,container):
        LOG("FlexData",DEBUG,"%r.__of__(%r)" % (self,container))
        w_self = ImplicitAcquisitionWrapper(aq_base(self), container)
        try:
            methods=getattr(w_self,self.subtype)
            LOG("FlexData",DEBUG,"methods=%r" % methods)
        except AttributeError,v:
            LOG("FlexData",DEBUG,
                "AttributeError:%r No subtype object for %r" % (self,
                                                                self.subtype))
            # A Dummy class is needed, so we can unconditionally strip of
            # one path element later in getPhysicalPath.
            methods=DummySubtypeClass()
        return ImplicitAcquisitionWrapper(aq_base(self),
                                          aq_base(methods).__of__(container))

    def getPhysicalPath(self):
        '''Returns a path (an immutable sequence of strings)
        that can be used to access this object again
        later, for example in a copy/paste operation.  getPhysicalRoot()
        and getPhysicalPath() are designed to operate together.
        For FlexDatas we need to forget about our subtype object.
        '''
        path = (self.getId(),)

        p = aq_parent(aq_parent(aq_inner(self)))
        if p is not None:
            path = p.getPhysicalPath() + path

        return path


Globals.InitializeClass(FlexData)
